#include "inland_config.h"
!
! #    #    ##       #    #    #
! ##  ##   #  #      #    ##   #
! # ## #  #    #     #    # #  #
! #    #  ######     #    #  # #
! #    #  #    #     #    #   ##
! #    #  #    #     #    #    # OFFLINE! :P
!
! Ported from Sage's IBIS as of 2011-06-02
! ---------------------------------------------------------------
#ifndef SINGLE_POINT_MODEL
#define INLAND_INFILE "conf/inland-grid.infile"
#else /* SINGLE_POINT_MODEL */
#define INLAND_INFILE "conf/inland-single_point.infile"
#endif

program main !main_offline!
! ---------------------------------------------------------------
! common modules
#ifndef SINGLE_POINT_MODEL
      use inland_ibispar
      use inland_ibisctl, only: jday, iday, imonth, iyear, iyear0, idailyout, &
                                imonthout, iyearout, isimveg, isimfire,       &
                                isimco2, nspinsoil, spinmax, eqyears, istep,  &
                                spincons, spinfrac, ipointout, ccmexist
      use inland_comatm, only: co2conc, o2conc
#else /* SINGLE_POINT_MODEL */
!     TODO: verify if there are variables that should be no longer used - fzm
      use inland_ibisparm, only: npoi, nsoilay, ndaypm, dtime, hvap, lbeg,   &
                                 lend, mpt, ndaypy, nlon, nlat, rhow, xres,  &
                                 yres
      use inland_ibisctl, only: jday, iday, imonth, iyear, iday0, imonth0,   &
                                iyear0, idailyout, imonthout, iyearout,      &
                                isimveg, isimfire, isimco2, nspinsoil, istep,&
                                spinmax, eqyears, spincons, spinfrac,        &
                                ipointout, ccmexist, nan
      use inland_com1d, only: firb, fsena, abupi, fvapa, abupd
      use inland_comatm, only: fira, co2conc, o2conc, solai, asuri, solad, asurd
      use inland_comhyd, only: gsuvap, grunof, gdrain, gtrans, ginvap
      use inland_comsoi, only: tg, hsoi, soihfl, poros, upsoil, tsoi, swilt, &
                               wsoi, rhosoi, csoi
      use inland_comsum, only: ayanpptot, wtot
      use inland_comveg, only: cbioltot, cbiortot, cbiowtot, chl, chs, chu,  &
                               sai, tco2mic, tu, ts, tl, lai, tco2root,      &
                               tnpptot, totlit, cbiol, cbior, cbiow, fl, fu, &
                               tgpptot, totcsoi, aleaf, awood, aroot,        &
                               totlail, totlaiu, wliql, wliqs, wliqu,        &
                               tneetot, clitll, clitlm, clitls, clitrl,      &
                               clitrm, clitrs, clitwl, clitwm, clitws,       &
                               topparl, topparu, falll
      use inland_comforc, only: xprec, xcld, xlati, xlin, xsin, xta, xqa, xua
#endif /* SINGLE_POINT_MODEL */
      use inland_comdiag, only: diagstart, diagend, ndiagpt, nfreq
      use inland_comwork, only: ndim4
      use inland_lsmmapib, only: lati, loni

      implicit none

! local variables
      integer icount,      & ! number of times ran2 is called (for restart)
              icountdum      !

      integer istyrd,      & ! first yr daily means exist (from precip file)
              istyrm,      & ! first yr monthly anoms exist (from file)
              iwest,       & ! 1st lon index for subset
              iy1,         & ! first year for year loop
              iy2,         & ! last year for year loop
              iyrdaily,    & ! year to start reading daily means
              iyrlast,     & ! last year of previous run (for restart)
              iyrmon,      & ! month year of previous run (for restart)
! The below variable has been inherited from CCSM3-ibis.
              imonthlast,  & ! last month of previous run (for restart)
              idiag,       & ! number of diagnostic files requested
              irestart,    & ! 0: normal mode 1: restart mode
              iyrrestart,  & ! Year to restart model
              jnorth,      & ! 1st lat index for subset
              niter,       & ! total number of time iterations per day
              nday,        & ! number of days since the start of the simulation
              nrun,        & ! # of years in this run
              plen,        & ! length of precipitation event in timesteps (see plens)
              plenmax,     & ! upper bound on plen
              plenmin,     & ! lower bound on plen

              seed,        & ! first value used to initialize the random # generator
              seed2,       & ! second value used to initialize the random # generator
              seed3(32),   & ! third value used to initialize the random # generator
              seed4,       & ! fourth value used to initialize the random # generator

              spin,        & ! counter for iterations used to spin up soilbgc
              soilcspin,   & ! 0: no spinup procedure for soil c  1: acceleration procedure used
              lun, ifile,  & ! file indices
              time,        & ! time of day since midnight (in seconds)
              i, n, ivar     ! loop indices

#ifdef SINGLE_POINT_MODEL
! Imbuzeiro: linenum is created to print the input files (xta, xqa, xua, xprec,
!            xsin, xlin) as an output
      integer linenum,     & ! line number
              daystart,    & ! index of when to start the day loop
              monthstart,  & ! index of when to start the month loop
              nbforc,      & ! number of timesteps of the simulation
              isoilforc,   & ! 0: fixed soil, 1: dynamic soil
              timel,       & ! local time of day since midnight (in seconds)
              nc             ! 0D specific loop indice

      real *8 rn,          & ! net radiation flux (short and long waves)
              pari,        & ! incoming PAR
              apar,        & ! APAR
              paro,        & ! outgoing PAR
              reflsw         ! reflected solar radiation

! Hewlley's variables as of june, 2007
! FIXME: this is no place for this kind of variables.
!       -fzm
      real*8  swnet,       & ! net shortware radiation
              lwnet,       & ! net longwave radiation
              vegt,        & ! vegetation canopy temperature
              avgsurft,    & ! average surface temperature
              surfalb,     & ! surface albedo
              soilwet,     & ! total soil wetness
              canopint,    & ! total canopy water storage
              rootmoist,   & ! root zone soil moisture
              canheat,     & ! canopy heat storage (J.m^-2)
              canheatl,    & ! last canopy heat storage (J.m^-2)
              delcanheat,  & ! change incanopy heat storage (J.m^-2)
              delsoilmoist,& ! change in soil moisture (kg.h2o.m^-2)
              soilmoistsum,& ! soil moisture in all layers (kg.h2o.m^-2)
              soilmoist(nsoilay), & ! soil moisture in which layer (kg.h2o.m^-2)
              soilmoistl,  & ! last soil moisture (kg.h2o.m^-2)
              autoresp,    & ! autotrophic respiration (kg.m^-2.s^-2)
              surfheat,    & ! surface heat storage (J.m^-2)
              delsurfheat, & ! change in surface heat storage (J.m^-2)
              surfheatl,   & ! last surface heat storage (J.m^-2)
              delsurfstor, & ! change in surface heat storage (J.m^-2)
              wtotl,    & ! total wather stored in soil, vegetation and snow
!                            ! at previous timestep (kg.h2o.m^-2.s^-1)
              totsoilcarb, & ! total soil carbon (kg.C.m^-2)
! FIXME: surfalb or albedo? are them the same? -fzm
              albedo,      & ! surface albedo
              solartot,    & ! total incoming radiation (dir+diff,vis+nIR)
! FIXME: avgsurft or soiltemp? are them the same? -fzm
              soiltemp,    & ! average layer soil temperature (K)
              undefined,   & ! undefined by albedo
              delwtot,     & ! change in total water stored (kg.h2o.m^-2.s^-1)
              fpar,        & ! absorbed fraction of PAR
              totlai,      & ! total ecosystem lai
              totlivbio,   & ! total living biomass (kg.C.m^-2)
              agb            ! above ground biomass
#endif /* SINGLE_POINT_MODEL */

      real*8  co2init,     & ! initial co2 concentration in mol/mol
              o2init,      & ! initial o2 concentration in mol/mol
              dran,        & ! random number from generator
              plens,       & ! length of precipitation event in seconds
              startp,      & ! time to start precipitation event (seconds since midnight)
              endp,        & ! time to end precipitation event (seconds since midnight)
              slope,       & ! rate of decrease of number of iteration for soil C spinup 
              snorth,      & ! north latitude for subsetting std grid
              ssouth,      & ! south latitude for subsetting std grid
              swest,       & ! west longitude for subsetting std grid
              seast,       & ! east longitude for subsetting std grid
              test           ! test on timestep
	      
      character*80     filen
      character*80   ::  directory='input/daily/prec.daily.'

#ifndef SINGLE_POINT_MODEL
! External
      real ran2              ! Function : random number generator
#endif /* SINGLE_POINT_MODEL */

! default values for subset grid
      data snorth, ssouth, swest, seast / 90., -90., -180., 180. /

! Ibis - CCSM3 variables
      logical rdlsf        ! set initial data for Antarctica, Greenland if true
      integer loopi,      &! index of little vector in big vector
              kpti,       &! start of vector segment
              kptj         ! end of vector segment

      ipointout=0

! ---------------------------------------------------------------
!                       j o b   c o n t r o l 
! ---------------------------------------------------------------
!
! open local input file called 'ibis.infile'
! and read in pertinent information
! TODO: add option to specify infile file name in command line.
! FIXME: Verify if every variable is allocated or it is needed another
!      'ibisalloc' call here.
      lun = 12
#ifdef SINGLE_POINT_MODEL
      linenum = 0
#endif /* SINGLE_POINT_MODEL */
      open (lun, status='old', file=INLAND_INFILE)
      read (lun,*) irestart
#ifndef SINGLE_POINT_MODEL
      read (lun,*) iyrrestart
#endif /* SINGLE_POINT_MODEL */
      read (lun,*) iyear0
      read (lun,*) nrun
      read (lun,*) iyrdaily
#ifndef SINGLE_POINT_MODEL
      read (lun,*) iyrmon
#endif /* SINGLE_POINT_MODEL */
      read (lun,*) soilcspin
      read (lun,*) iyearout
      read (lun,*) imonthout
      read (lun,*) idailyout
      read (lun,*) isimveg
      read (lun,*) isimfire
      read (lun,*) isimco2
#ifdef SINGLE_POINT_MODEL
      read (lun,*) isoilforc
#endif /* SINGLE_POINT_MODEL */
      read (lun,*) co2init
      read (lun,*) o2init
      read (lun,*) dtime
      read (lun,*) idiag
      read (lun,*,end=99) snorth
      read (lun,*,end=99) ssouth
      read (lun,*,end=99) swest
      read (lun,*,end=99) seast

99    close (lun)

! we are not coupled. ccmexist (from ibisctl) is 0.
      ccmexist = 0

! tell user about the simulation
      write (*,*) ' '
      write (*,*) '**************************************'
      write (*,*) '* InLand - Surface model for the MBSCG'
      write (*,*) '**************************************'
      write (*,*) '* based on IBIS v2.6b4'
      write (*,*) '* Version ',PACKAGE_VERSION,', August 2012'
#ifdef SINGLE_POINT_MODEL
      write (*,*) '* 0D: Single Point Model version.'
#endif /* SINGLE_POINT_MODEL */
      write (*,*) '**************************************'
      write (*,*) ' '
      if (irestart .eq. 1) then
         write (*,*) 'INFO: running in restart mode'
#ifdef SINGLE_POINT_MODEL
         write (*,*) 'ERROR: Single Point Model (0D) does NOT support restart.'
         stop 'Please turn off restart (make it 0) on config file.'
#endif /* SINGLE_POINT_MODEL */
      end if
      write (*,*) ' '
      write (*,9000) nrun
      write (*,*) ' '
      write (*,9010) xres, yres
      write (*,9020) nlon, nlat
      write (*,*) ' '

9000  format (1x,'INFO: length of this simulation (years)   : ',i8)
9005  format (1x,'INFO: year to begin using anomalies       : ',i8)
9006  format (1x,'INFO: number of iterations per day        : ',i8)
9010  format (1x,'INFO: model lon, lat resolution (degrees) : ',2f8.2)
9020  format (1x,'INFO: model domain (nlon x nlat)          : ',i3,' by ',i3)  
9030  format (1x,'INFO: last year run in this sequence      : ',i8)

      if (iyrdaily.eq.iyrmon .and. iyrdaily.ne.9999) then
          write(*,*) 'INFO: Its impossible to run because iyrdaily:',iyrdaily,' equal irymon:', iyrmon
          stop
      endif

! ---------------------------------------------------------------
!                     t i m e   c o n t r o l 
! ---------------------------------------------------------------
! determine the number of timesteps per day
      niter = int(86400.0 / dtime)

! test on length of the time step and total number of iteration per day
      test = dmod(dble(86400.0), dtime)

      if (test .gt. 1.e-20) then
         write (*,*) 'dtime ', dtime, ' should be divisible into 86400'  
         stop
      else
         write (*,9006) niter
      end if

! check for restart conditions and set "iyrlast" accordingly, open
! ibis.annual in append mode
      if (irestart .eq. 1) then
         write(filen,'(A16,I4,A4)')'output/yearsrun.',iyrrestart,'.dat'
         open(13, status='old', file=filen, err=9050)
         read(13,*) iyrlast
! those variables to turn possible restart of ran2 subroutine
         read(13,*) seed
         read(13,*) seed2
         read(13,*) seed3
         read(13,*) seed4
         close(13)
         goto 9059
  9050   write(*,*) 'warning: ibis.infile restart flag=1, indicating restart, but output/yearsrun.',iyrrestart,'.dat not found'
         stop
         write(*,*) 'beginning from ', iyear0
         iyrlast = iyear0 - 1
         open(20,file='ibis.out.global',status='unknown',access='append')
         open(30,file='ibis.out.vegtype',status='unknown',access='append')
      else
         iyrlast = iyear0 - 1
9059  end if

      write (*,9030) nrun + iyrlast
      write (*,*) ' '

!   Later in this point we can parallelize the code to run lbeg:lend subsets of
! the model for each timestep!
!   Now we just set lbeg=1 and lend=npoi (given from ibisparm)
      lbeg=1
      lend=npoi
      mpt=lend-lbeg+1

! ---------------------------------------------------------------
!                   i n i t i a l i z a t i o n
! ---------------------------------------------------------------
! Allocate variables needed by rd_param and readit.
      call inland_prealloc

! call RD_PARAM to read in all parameter files
      call rd_param

! initial concentration of co2, o2
      co2conc = co2init
      o2conc  = o2init

! read global boundary condition datasets
      call readit(isimveg,snorth,ssouth,swest,seast,iwest,jnorth)

!-----------------
! Data structures allocation: call ibisalloc
! TODO: maybe we need setting a 'mpt' variable, according to ibisini subroutine.
!      Sometimes you can see 'var(mpt)' and other 'var(lbeg:lend)' to the same
!      sized variables in SAGE IBIS 'var(npoi)'.
! ibisalloc -must- be after readit() for nlonsub+nlatsub to be already defined
! at the time of allocation.
      call ibisalloc

#ifdef SINGLE_POINT_MODEL
! read forcing data
      call readforc(isoilforc,nbforc)

! FIXME: 'test' variable. No clue what it is for, this is a badly written code.
      test = nbforc
#endif /* SINGLE_POINT_MODEL */

! map coupled model variables based on variables read from data files
      call ibismap_offline(imonthlast, rdlsf, loopi, kpti, kptj, ipointout)

! check if diagnostic output is requested, if so read info from 'diag.infile'
! TODO: not supported yet, can be added as a later feature. Just keep '0' at
!       conf/inland-<model>.infile for 'idiag'.
      !if (idiag .ne. 0) call inidiag(idiag)

#ifndef SINGLE_POINT_MODEL
! preliminary analysis of climate data
      if (irestart.eq.0 .or. isimveg.eq.0) then
         call climanl
      end if

      if (iyrdaily .le. iyrlast+nrun) then
         write(filen,'(A23,I4,A3)')trim(directory),iyrdaily,'.nc'
         call inird(filen,istyrd)
	 if (istyrd .le. 0) istyrd = 9999
      end if

!    If first year of this run/restart is an anomaly year, then read in 
! anomalies for month 12 of previous year and month 1 of this year
! (note: rdanom reads imonth+1) 
      iy2 = iyrlast + nrun
#endif /* SINGLE_POINT_MODEL */

! initialize the model
      call initialib(isimveg, irestart, iyrlast, imonthlast, rdlsf, lati, loni)

#ifndef SINGLE_POINT_MODEL
! initialize random number generator seed
      if(irestart.eq.0) then
         seed = -1
         seed2 = 123456789
         seed3 = 0
         seed4 = 0
         dran = ran2 (seed,seed2,seed3,seed4)
      endif
#endif /* SINGLE_POINT_MODEL */

! ---------------------------------------------------------------
!              s t a r t   o f   t i m e   l o o p s
! ---------------------------------------------------------------
      write (*,*) ' '
      write (*,*) '********************************'
      write (*,*) '* start of the IBIS simulation *'
      write (*,*) '********************************'
      write (*,*) ' '
#ifdef SINGLE_POINT_MODEL
! Initialize the output file from outside the time loops -fzm
! Write half-hourly output
! Imbuzeiro: Added the variable names in the output file
      open(40,file='output/single_point-output.csv',status='unknown')
      write(40,9199) 'time','Swnet','Lwnet','Qle','Qh','Qg','DelCanHeat',   &
                     'DelSurfHeat','Evap','Qs','Qrec','Qsb','Qt',           &
                     'DelSoilMoist','DelSurfStor','DelIntercept',           &
                     'WaterTableD','VegT','BaresoilT','AvgSurfT','Albedo',  &
                     'SurfStor','fPAR','SoilMoist1','SoilMoist2',           &
                     'SoilMoist3','SoilMoist4','SoilMoist5','SoilMoist6',   &
                     'SoilTemp1','SoilTemp2','SoilTemp3','SoilTemp4',       &
                     'SoilTemp5','SoilTemp6','SoilWet','Ecanop','Tveg',     &
                     'Esoil','Ewater','RootMoist','CanopInt','GPP',         &
                     'NPP','NEE','AutoResp','HeteroResp','ANPP','BgResp',   &
                     'LitterFall','CarbPools1','CarbPools2','CarbPools3',   &
                     'CarbPools4','CarbPools5','CarbPools6','CarbPools7',   &
                     'CarbPools8','CarbPools9','CarbPools10','CarbPools11', &
                     'CarbPools12','CarbPools13','TotLivBiom','AbvGrndWood',&
                     'LAI','Tair','Qair','Wind','Rainf','Psurf','Swdown',   &
                     'Lwdown','CO2air'
9199  format (74(a,x))
#endif /* SINGLE_POINT_MODEL */

! reset elapsed number of days, accumulating past days if restart mode
      nday = 0

      if (irestart .eq. 1) then
         do 150 iyear = iyear0, iyrlast
            nday = nday + 365
            if (mod(iyear,4).eq.0) then
               if (mod(iyear,100).ne.0) then
                  nday = nday + 1
               elseif (mod(iyear/100,4).eq.0) then
                  nday = nday + 1
               end if
            end if
150      continue
      end if

!---------------------- YEAR YEAR YEAR -------------------------------------
! start of yearly loop
!---------------------- YEAR YEAR YEAR -------------------------------------
      iy1 = iyrlast + 1
      iy2 = iyrlast + nrun
      do 200 iyear = iy1, iy2
! reset julian date
         jday = 0

!    Determine the calendar for this year.
!    Leap years occur every calendar year that is divisible by 4 (such as
! 1980, 1984, etc.) *except* for years which begin centuries not divisible
! by 400
!    For example, 1700, 1800, and 1900 are *not* leap years, but 1600
! and 2000 *are* leap years
! NOTICE: CCSM3 version was using individual ndaypy on soilbgc, sumnow, sumyear,
!        and wyearly subroutines. The value was fixed at 365. This might bring
!        trouble as we fixed back to SAGE IBIS behavior.
         ndaypm(2) = 28
         ndaypy = 365

! 0D IBIS does not consider leap years.
#ifndef SINGLE_POINT_MODEL
         if (mod(iyear,4).eq.0) then
            if (mod(iyear,100).ne.0) then
               ndaypm(2) = 29
               ndaypy = 366
            elseif (mod(iyear/100,4).eq.0) then
               ndaypm(2) = 29
               ndaypy = 366
            end if
         end if
#endif /* SINGLE_POINT_MODEL */

!---------------------- MONTH MONTH MONTH ----------------------------------
! start of monthly loop
!---------------------- MONTH MONTH MONTH ----------------------------------
#ifdef SINGLE_POINT_MODEL
! Observed, step data, might not start on first month of year.
         if (iyear.eq.iy1) then 
            monthstart = imonth0
         else
            monthstart = 1
         endif
         do 210 imonth = monthstart, 12
! rdanom is not called on single point model.
#else /* SINGLE_POINT_MODEL */
         do 210 imonth = 1, 12
!            call rdanom(iyranom,nanom,iy2,istyrm,iwest,jnorth)
#endif /* SINGLE_POINT_MODEL */

!------------------------- DAY DAY DAY -------------------------------------
! start of daily loop
!------------------------- DAY DAY DAY -------------------------------------
#ifdef SINGLE_POINT_MODEL
! Observed, step data, might not start on first day of the month for the first
!   month.
            if ( (iyear.eq.iy1).and.(imonth.eq.imonth0) ) then
               daystart = iday0
            else
               daystart = 1
            endif
            
            do 220 iday = daystart, ndaypm(imonth)
#else /* SINGLE_POINT_MODEL */
            do 220 iday = 1, ndaypm(imonth)
#endif /* SINGLE_POINT_MODEL */

! update user on model progress once a day
               write (STDOUT,9100,advance='no') char(13), char(13), iyear, imonth, iday
#ifdef IFORT_NOFLUSH_BUG
               close(STDOUT)
               open(STDOUT)
#endif
#ifdef PGF90_NOFLUSH_BUG
               call flush(STDOUT)
#endif
9100           format (a,79x,a,'Running now: ',i4.4,'/',i2.2,'/',i2.2)
9101           format (a,79x,a,'Running now: ',i4.4,'/',i2.2,'/',i2.2,a)
9102           format (a,79x,a,'Completed ',i4.4,'/',i2.2,'/',i2.2)

! update elapsed number of days and julian date
               nday = nday + 1
               jday = jday + 1

#ifdef SINGLE_POINT_MODEL
               call daily(test,nbforc)
#else /* SINGLE_POINT_MODEL */
! get daily means (if requested)
! and determine today's climatic conditions

! TODO: port the weather generator!
!      maybe we need to sed the kpti-kptj interval to allow parallelizing the
!      weather generator. by now, we just use lbeg:lend but keep your eyes open!
               if (iyear.ge.iyrdaily) then
                  call rdday(jday,istyrd,iwest,jnorth)
                  call daily(seed,seed2,seed3,seed4,1,iyrlast,nrun)
               else
                  if (iyear.ge.iyrmon) then
                     call rdmon(iyear, iwest, jnorth)
                     call dailymon(seed,seed2,seed3,seed4,0,iyrmon,iyrlast,nrun)
                  else
                     call daily(seed,seed2,seed3,seed4,0,iyrlast,nrun)
                  endif
               end if
#endif /* SINGLE_POINT_MODEL */

! determine the daily vegetation cover characteristics
               call pheno(kpti, kptj)

! call soil biogeochemistry model
!
! soil spin up procedure calls soil bgc model spincons times
! for each time step to spin up the process (accelerate C & N pools)
               spinfrac  = 0.75     ! fraction of nspinsoil time used to
!                                   ! spin up soil at maximum spin up rate
!
               spincons  = 40.0     ! number of times soilbgc subroutine is
!                                   ! called for each day in simulation during
!                                   ! acceleration procedure
!
               eqyears   = 50       ! years used past spin up to slowly bring
!                                   ! entire soil to a steady equilibrium
!
               nspinsoil=iyear0+150 ! year of simulation that soil c reaches equilibrium

               if (soilcspin .eq. 1) then
                  if ((iyear-iyear0) .le. (spinfrac*(nspinsoil-iyear0-eqyears))) then
                     spinmax = int(spincons)
                  elseif ((iyear-iyear0) .lt. (nspinsoil-iyear0-eqyears)) then
                     slope   = spincons / ((nspinsoil - iyear0 - eqyears) -   &
                               (spinfrac * (nspinsoil - iyear0 - eqyears)))
                     spinmax = int(spincons - (slope * ((iyear - iyear0) -    &
                               (spinfrac * (nspinsoil - iyear0 - eqyears)))))
                     spinmax = max(spinmax,1)
                  else
                     spinmax = 1
                  endif
               else 
                  spinmax = 1
               endif

               do 230 spin = 1, spinmax
                  call soilbgc(spin,kpti,kptj)
230            continue

#ifndef SINGLE_POINT_MODEL
! determine the length of a precipitation event (between 4 and 24 hours),
! and time to start and end precipitation event. plen is in timesteps, while
! plens, startp, and endp are in seconds
               plenmin = 1 + int((4.0 * 3600. - 1.) / dtime)
               plenmax = max(int(24.0 * 3600. / dtime), plenmin)
               plen = min(plenmax,int(plenmin+ran2(seed,seed2,seed3,seed4) * (plenmax-plenmin+1)))
               plens   = dtime * plen
               startp  = dtime * min(niter-plen, int(ran2(seed,seed2,seed3,seed4)*(niter-plen+1)))
               endp    = startp + plens
#endif /* SINGLE_POINT_MODEL */

!---------------------- HOUR HOUR HOUR -------------------------------------
! start of hourly loop
! if SINGLE_POINT_MODEL (0D), then this is half-hourly loop.
!---------------------- HOUR HOUR HOUR -------------------------------------
               do 240 istep = 1, niter

! calculate the time of day since midnight (in seconds)
                  if (dtime - int(dtime) .ne. 0) then
                     stop "ERROR: Time slice is not integer!"
                  endif
                  time = (istep - 1) * int(dtime)

#ifdef SINGLE_POINT_MODEL
! determine climatic conditions for the given timestep
                  call diurnal(time, jday, test, nbforc)
! call the land surface model
                  call lsxmain(loopi, kpti, kptj, test, nbforc, isoilforc)
#else /* SINGLE_POINT_MODEL */
! determine climatic conditions for the given timestep
                  call diurnal(time, jday, plens, startp, endp, seed, seed2,seed3,seed4)
! call the land surface model
                  call lsxmain(loopi, kpti, kptj)
#endif /* SINGLE_POINT_MODEL */

! accumulate some variables every timestep
                  call sumnow(kpti,kptj)
                  call sumday  (time, loopi, kpti, kptj)
                  call summonth(time, loopi, kpti, kptj)
                  call sumyear (time, loopi, kpti, kptj)

! write out diagnostics
                  if (idiag .ne. 0) then
                     do 250 i = 1, idiag
                        if (iyear.ge.diagstart(i) .and. iyear.le.diagend(i) &
                            .and. ndiagpt(i).ge.1) then  
                           if (mod(istep,nfreq(i)).eq.0) then
                              call wdiag(i, istep)
                           end if
                        end if
250                  continue
                  end if

#ifdef SINGLE_POINT_MODEL
! energy budget of the surface
                  rn = solad(1,1) * (1-asurd(1,1)) + &
                       solad(1,2) * (1-asurd(1,2)) + &
                       solai(1,1) * (1-asuri(1,1)) + &
                       solai(1,2) * (1-asuri(1,2)) + fira(1) - firb(1)

                  pari = (solad(1,1) + solai(1,1)) * 4.59
                  apar = (topparl(1) + topparu(1)) + 4.59
                  paro = (solad(1,1) * asurd(1,1) + solai(1,1)*asuri(1,1))*4.59

                  swnet = solad(1,1) * (1-asurd(1,1)) + &
                          solad(1,2) * (1-asurd(1,2)) + &
                          solai(1,1) * (1-asuri(1,1)) + &
                          solai(1,2) * (1-asuri(1,2))

                  lwnet = (fira(1) - firb(1))

                  reflsw = solad(1,1)*asurd(1,1) + solad(1,2)*asurd(1,2) + &
                           solai(1,1)*asuri(1,1) + solai(1,2)*asuri(1,2)

                  solartot = solad(1,1) + solad(1,2) + solai(1,1) + solai(1,2)

! Imbuzeiro: Created to replace the NaN values to zero, because
!            in time whitout radiation the ibis write NaN and I
!            excluded the NaN to used the output file in my
!            netcdf program (.txt to .nc)
! In accordance to Hewlley Imbuzeiro, when solartot is 0, albedo must be an 
! 'expected' NaN (or -999.99 defined by the 'nan' variable on ibisctl) - fzm
                  if (solartot.ne.0) then
                     albedo = reflsw / solartot
                  else
                     albedo = nan
                  endif

                  vegt = ((chl*tl(1)) + (chs*ts(1)) + (chu*tu(1))) / &
                         (chs+chl+chu)

                  avgsurft = (tu(1) + tl(1) + tg(1)) / 3

! Surfalb is -999.99 (expected NaN) when solad+solai = 0 (no solar incidence)
                  if ((solad(1,1)+solai(1,1)).ne.0.) then
                     surfalb = ((solad(1,1)*(1-asurd(1,1))) + (solai(1,1) * &
                               (1-asuri(1,1)))) / (solad(1,1) + solai(1,1))
                  else
                     surfalb = nan
                  endif

                  soilwet = ((wsoi(1,1) - swilt(1,1) * hsoi(1) * poros(1,1)) + &
                            (wsoi(1,2) - swilt(1,2) * hsoi(2) * poros(1,2)) +  &
                            (wsoi(1,3) - swilt(1,3) * hsoi(3) * poros(1,3)) +  &
                            (wsoi(1,4) - swilt(1,4) * hsoi(4) * poros(1,4)) +  &
                            (wsoi(1,5) - swilt(1,5) * hsoi(5) * poros(1,5)) +  &
                            (wsoi(1,6) - swilt(1,6) * hsoi(6) * poros(1,6))) / &
                            ((1-swilt(1,1) * poros(1,1) * hsoi(1)) +           &
                            (1-swilt(1,2) * poros(1,2) * hsoi(2)) +            &
                            (1-swilt(1,3) * poros(1,3) * hsoi(3)) +            &
                            (1-swilt(1,4) * poros(1,4) * hsoi(4)) +            &
                            (1-swilt(1,5) * poros(1,5) * hsoi(5)) +            &
                            (1-swilt(1,6) * poros(1,6) * hsoi(6)))

                  canopint = (wliqu(1) * fu(1) * 2 * lai(1,2) + &
                              wliqs(1) * fu(1) * 2 * sai(1,2) + &
                              wliql(1) * fl(1) * 2 * (lai(1,1) + sai(1,1)))

                  rootmoist = (upsoil(1,1) + gsuvap(1))
                  totsoilcarb = tco2root(1) + tco2mic(1)
                  delcanheat = (swnet + lwnet - fvapa(1)*hvap - fsena(1) + &
                                soihfl(1)) * 3600

! Imbuzeiro: Correct the soilmoisture equation
! Turned the code into a do-loop for simplicity - fzm
                  soilmoistsum=0.
                  do 260 i=1,nsoilay
                     soilmoist(i) = (wsoi(1,i) * poros(1,i) * rhow * hsoi(i))
                     soilmoistsum = soilmoistsum + soilmoist(i)
260               continue

                  delsoilmoist = soilmoistsum - soilmoistl
                  soilmoistl = soilmoistsum

                  surfheat=0.
                  do 270 i=1,nsoilay
                     surfheat=surfheat+(csoi(1,i)*rhosoi(1,i)*tsoi(1,i)*hsoi(i))
270               continue

                  delsurfheat = surfheat - surfheatl
                  surfheatl = surfheat
                  
                  delwtot = wtot(1) - wtotl
                  wtotl = wtot(1)

                  autoresp = tgpptot(1) - tnpptot(1)
                  totsoilcarb = totcsoi(1) + totlit(1)

! Imbuzeiro: Created to replace the NaN values to zero, because
!            in time whitout radiation the ibis write NaN and I
!            excluded the NaN to used the output file in my
!            netcdf program (.txt to .nc)
! In accordance to Hewlley Imbuzeiro, when solartot is 0, albedo must be an 
! 'expected' NaN (or -999.99 defined by the 'nan' variable on ibisctl) - fzm
                  if (solad(1,1).ne.0) then
                     fpar = (solad(1,1)*abupd(1) + solai(1,1)*abupi(1)) / &
                            (solad(1,1) + solai(1,1))
                  else
                     fpar = nan
                  endif

! FIXME: make a DO loop for totlivbio, agb, cbio?tot all at once -fzm
                  totlivbio = cbiol(1,1) + cbior(1,1) + cbiow(1,1) + &
                              cbiol(1,2) + cbior(1,2) + cbiow(1,2) + &
                              cbiol(1,3) + cbior(1,3) + cbiow(1,3) + &
                              cbiol(1,4) + cbior(1,4) + cbiow(1,4) + &
                              cbiol(1,5) + cbior(1,5) + cbiow(1,5) + &
                              cbiol(1,6) + cbior(1,6) + cbiow(1,6) + &
                              cbiol(1,7) + cbior(1,7) + cbiow(1,7) + &
                              cbiol(1,8) + cbior(1,8) + cbiow(1,8) + &
                              cbiol(1,9) + cbior(1,9) + cbiow(1,9) + &
                              cbiol(1,10) + cbior(1,10) + cbiow(1,10) + &
                              cbiol(1,11) + cbior(1,11) + cbiow(1,11) + &
                              cbiol(1,12) + cbior(1,12) + cbiow(1,12)

                  agb = cbiol(1,1) + cbiow(1,1) + cbiol(1,2) + cbiow(1,2) + &
                        cbiol(1,3) + cbiow(1,3) + cbiol(1,4) + cbiow(1,4) + &
                        cbiol(1,5) + cbiow(1,5) + cbiol(1,6) + cbiow(1,6) + &
                        cbiol(1,7) + cbiow(1,7) + cbiol(1,8) + cbiow(1,8) + &
                        cbiol(1,9) + cbiow(1,9) + cbiol(1,10) + cbiow(1,10) + &
                        cbiol(1,11) + cbiow(1,11) + cbiol(1,12) + cbiow(1,12)

                  cbioltot(1) = cbiol(1,1) + cbiol(1,2) + cbiol(1,3) + &
                                cbiol(1,4) + cbiol(1,5) + cbiol(1,6) + &
                                cbiol(1,7) + cbiol(1,8) + cbiol(1,9) + &
                                cbiol(1,10) + cbiol(1,11) + cbiol(1,12)

                  cbiortot(1) = cbior(1,1) + cbior(1,2) + cbior(1,3) + &
                                cbior(1,4) + cbior(1,5) + cbior(1,6) + &
                                cbior(1,7) + cbior(1,8) + cbior(1,9) + &
                                cbior(1,10) + cbior(1,11) + cbior(1,12)

                  cbiowtot(1) = cbiow(1,1) + cbiow(1,2) + cbiow(1,3) + &
                                cbiow(1,4) + cbiow(1,5) + cbiow(1,6) + &
                                cbiow(1,7) + cbiow(1,8) + cbiow(1,9) + &
                                cbiow(1,10) + cbiow(1,11) + cbiow(1,12)

                  totlai = totlaiu(1) + totlail(1)

! Imbuzeiro: For the LBA-DMIP protocol I did a spinup but I only wanted
!            write the last 3 years. This information changes for other
!            sites and user specifications
                  if ( (iyear.ge.2001).and.(iyear.le.2003) ) then
                     linenum = linenum + 1
                     write(40,9200) linenum, swnet, lwnet, -fvapa(1)*hvap,     &
                                    -fsena(1), soihfl(1), delcanheat, nan,     &
                                    -fvapa(1), grunof(1), nan, gdrain(1),      &
                                    gdrain(1)+grunof(1), delsoilmoist, nan,    &
                                    nan, nan, vegT,tsoi(1,1), nan, albedo, nan,&
                                    fpar,(soilmoist(i),i=1,nsoilay),           &
                                    (tsoi(1,i),i=1,nsoilay), soilwet,ginvap(1),&
                                    gtrans(1), gsuvap(1), nan,nan, canopint,   &
                                    tgpptot(1)*0.012,tnpptot(1)*0.012,         &
                                    -tneetot(1)*0.012,autoresp*0.012,          &
                                    tco2mic(1)*0.012,ayanpptot(1), nan,        &
                                    falll(1),cbioltot(1), cbiortot(1),         &
                                    cbiowtot(1),clitll(1), clitlm(1),          &
                                    clitls(1),clitrl(1), clitrm(1), clitrs(1), &
                                    clitwl(1), clitwm(1), clitws(1), agb,      &
                                    totlivbio, cbiowtot(1), totlai,            &
                                    xta(1,linenum)+273.15, xqa(1,linenum),     &
                                    xua(1,linenum), xprec(1,linenum)/1800, nan,&
                                    xsin(1,linenum), xlin(1,linenum), nan
                  endif                       
9200              format(i10,x,74(f25.16,x))

                  test = test - 1
! end of half-hourly loop
                  if (test.eq.0.) goto 9999
#endif /* SINGLE_POINT_MODEL */

! ---------------------------------------------------------------
!               e n d   o f   t i m e   l o o p s
! ---------------------------------------------------------------

!---------------------- HOUR HOUR HOUR -------------------------------------
! end of the hourly loop
!---------------------- HOUR HOUR HOUR -------------------------------------
240            continue

#ifndef SINGLE_POINT_MODEL
! write out daily output
               if (idailyout.eq.1) then
                  write (STDOUT,9101,advance='no') char(13), char(13), iyear, &
                                                   imonth, iday,              &
                                                   ' *writing daily output*'
#ifdef IFORT_NOFLUSH_BUG
                  close(STDOUT)
                  open(STDOUT)
#endif /* IFORT_NOFLUSH_BUG */

#ifdef PGF90_NOFLUSH_BUG
                  call flush(STDOUT)
#endif /* PFG90_NOFLUSH_BUG */

                  call wdaily(jday) 
               endif
#endif /* SINGLE_POINT_MODEL */
!------------------------- DAY DAY DAY -------------------------------------
! end of the daily loop
!------------------------- DAY DAY DAY -------------------------------------
220         continue
!
! write out monthly output (use 1st day of this month because of GrADS bug)
#ifndef SINGLE_POINT_MODEL
            if (imonthout.eq.1) then
               write (STDOUT,9101,advance='no') char(13), char(13), iyear, &
                                                imonth, iday-1,            &
                                                ' *writing monthly output*'
#ifdef IFORT_NOFLUSH_BUG
               close(STDOUT)
               open(STDOUT)
#endif /* IFORT_NOFLUSH_BUG */

#ifdef PGF90_NOFLUSH_BUG
               call flush(STDOUT)
#endif /* PGF90_NOFLUSH_BUG */

               call wmonthly(nday-ndaypm(imonth)+1)
            endif
#endif /* SINGLE_POINT_MODEL */
!---------------------- MONTH MONTH MONTH ----------------------------------
! end of the monthly loop
!---------------------- MONTH MONTH MONTH ----------------------------------
210      continue

#ifndef SINGLE_POINT_MODEL
! recalculate bioclimatic parameters if using anomalies
         if (iyear.ge.iyrdaily) call climanl2
#endif /* SINGLE_POINT_MODEL */

! NOTICE: Although Hewlley's version has CO2 commented out, you can disable it
!        just by not setting isimco2=1 on the infile.
! get new o2 and co2 concentrations for this year
         if (isimco2.eq.1) call co2(co2init, co2conc)

! perform vegetation dynamics
         if (isimveg.ne.0) call dynaveg

! calculate simple annual diagnostic variables for the globe
         call gdiag

! calculate simple annual diagnostic variables for each vegetation type
         call vdiag

#ifndef SINGLE_POINT_MODEL
! write out annual output
         if (iyearout.eq.1) then
            write (STDOUT,9101,advance='no') char(13),char(13),iyear,imonth-1, &
                                        iday-1, ' *writing yearly output*'
#ifdef IFORT_NOFLUSH_BUG
            close(STDOUT)
            open(STDOUT)
#endif /* IFORT_NOFLUSH_BUG */

#ifdef PGF90_NOFLUSH_BUG
            call flush(STDOUT)
#endif /* PGF90_NOFLUSH_BUG */

            call wyearly(nday)
         endif

! write restart files
         write (STDOUT,9101,advance='no') char(13), char(13), iyear, imonth-1, &
                                     iday-1, ' *writing restart data*'
#ifdef IFORT_NOFLUSH_BUG
         close(STDOUT)
         open(STDOUT)
#endif /* IFORT_NOFLUSH_BUG */

#ifdef PGF90_NOFLUSH_BUG
         call flush(STDOUT)
#endif /* PGF90_NOFLUSH_BUG */

! Now just restart for dinamic vegetation (option 1 only)
        if(isimveg.ge.1) call wrestart(nday, imonthlast)

#endif /* SINGLE_POINT_MODEL */

! update iyrlast value and file yearsrun.dat
         iyrlast = iyrlast + 1
         write(filen,'(A16,I4,A4)')'output/yearsrun.',iyrlast,'.dat'
         open(12,file=filen,status='unknown')
         write(12,*) iyrlast
! uptade variables to turn possible restart of ran2 subroutine
         write(12,*) seed
         write(12,*) seed2
         write(12,*) seed3
         write(12,*) seed4
         close(12)
         write (STDOUT,9102) char(13), char(13), iyear, imonth-1, iday-1
!---------------------- YEAR YEAR YEAR -------------------------------------
! end of the yearly loop
!---------------------- YEAR YEAR YEAR -------------------------------------
200   continue

#ifdef SINGLE_POINT_MODEL
9999  continue
! If jumped here due to test = 0, then force calling the final g/vdiag.
      if (test.eq.0) then
         call gdiag
         call vdiag
      endif
#endif /* SINGLE_POINT_MODEL */

! end of the simulation
      write (*,*) ' '
      write (*,*) '*** end of run ***'
      stop
end program main !main_offline
