#include "inland_config.h"
! ---------------------------------------------------------------------
subroutine readit(isimveg,snorth,ssouth,swest,seast,iwest,jnorth)
! ---------------------------------------------------------------------
! reads in initialization files and initializes some fields
      use inland_ibispar
      use inland_combcs
      use inland_comsoi
      use inland_comveg
      use inland_comwork

      implicit none

! Arguments
      integer isimveg, & ! dynamic vegetation (1) or static (0)
              iwest, jnorth

      real*8 snorth, ssouth, swest, seast

! Local variables
      integer  istat,  & ! netcdf error flag
           i, j, ntime,& ! loop indices
           jj, ii,     &
           ndim,       & ! number of dimensions
           nlpoints      ! number of land points

      real*8 xlat

      integer jsouth, ieast

! xmask(nlon,nlat) ::> equivalence --> pointer 
! cant use equivalence ( xmask(1,1), work(1) ) to xmask with allocatable arrays
!   The work variable is ndim2 which is nlon*nlat size. Here, we just declare
! xmask as (nlon,nlat)-dimensioned and use it on a new memory space.
!   If there's still problem with that approach, we will have to allocate work 
! statically. As of yet, nlon and nlat are compile-time options and this
! will not be a problem. Even if we were to get nlon and nlat from input config
! file (inland.conf/inland.infile/ibis.infile), this would already have been 
! done.
      real*8 xmask(nlon,nlat)

      character*80 filen

      integer istart(4), icount(4) ! for reading vars

! ---------------------------------------------------------------------
      data istart / 1,1,1,1 /, icount / nlon,nlat,1,1 /

! 2-d surface and vegetation arrays
      icount(3) = 1

#ifndef SINGLE_POINT_MODEL
! land mask, latitudes, and longitudes
      filen = 'input/surta.nc'
      aname = 'surta'
      call readvar(filen,aname,'level',istart,icount,xmask,lonscale,latscale,cdummy(1),cdummy(ndim4),istat)
      if (istat.lt.0) then
         write (*,9000)
         print *, 'while reading surta'
         stop 1
      end if

      if (abs(abs(lonscale(1)-lonscale(2))-xres).gt.0.001 .or. abs(abs(latscale(1)-latscale(2))-yres).gt.0.001) then
         write (*,9000)
         write(*,*) 'resolution mismatch!'
         write(*,*) 'xres, yres in include/inland_compar.h = ', xres, yres
         write(*,*) 'xres, yres from input/surta.nc = ', &
                    abs(lonscale(1)-lonscale(2)), abs(latscale(1)-latscale(2))
         stop 1
      end if
#endif /* SINGLE_POINT_MODEL */

! subset the grid if not default whole grid
      if (snorth.lt.latscale(1) .or. ssouth.gt.latscale(nlat) .or. &
          swest.gt.lonscale(1) .or.  seast.lt.lonscale(nlon)) then
         jnorth = 0
        
         if (snorth .lt. (latscale(nlat)+latscale(nlat-1))/2.) then
            jnorth = nlat
         elseif (snorth .ge. (latscale(1)+latscale(2))/2.) then
            jnorth = 1
         else
            do 1 j = nlat-1,1,-1
               if (snorth .ge. (latscale(j)+latscale(j+1))/2.) jnorth = j
1           continue
         end if
         jsouth = 0
         if (ssouth .lt. (latscale(nlat)+latscale(nlat-1))/2.) then
            jsouth = nlat
         elseif (ssouth .ge. (latscale(1)+latscale(2))/2.) then
            jsouth = 1
         else
            do 2 j = nlat-1,1,-1
               if (ssouth .ge. (latscale(j)+latscale(j+1))/2.) jsouth = j
2           continue
         end if
!
         iwest = 0
         if (swest .lt. (lonscale(1)+lonscale(2))/2.) then
            iwest = 1
         elseif (swest .ge. (lonscale(nlon)+lonscale(nlon-1))/2.) then
            iwest = nlon
         else
            do 3 i = 2, nlon
               if(swest .ge. (lonscale(i)+lonscale(i-1))/2.) iwest=i
3           continue
         end if
         ieast = 0
         if (seast .lt. (lonscale(1)+lonscale(2))/2.) then
            ieast = 1
         elseif (seast .ge. (lonscale(nlon)+lonscale(nlon-1))/2.) then
            ieast = nlon
         else
            do 4 i = 2, nlon
               if(seast .ge. (lonscale(i)+lonscale(i-1))/2.) ieast=i
4           continue
         end if
         nlonsub = ieast - iwest + 1
         nlatsub = jsouth - jnorth + 1
         istart(1) = iwest
         icount(1) = nlonsub
         istart(2) = jnorth
         icount(2) = nlatsub
      else
         iwest = 1
         ieast = nlon
         jnorth = 1
         jsouth = nlat
         nlonsub = nlon
         nlatsub = nlat
      end if

! The lmask(nlonsub,nlatsub) or rather lmask(plon,plat) must be allocated here!
! Remember plona => nlonsub and plata => nlatsub on inland_prealloc!
      allocate(lmask(plona,plata))

! First, we check up the npoi amount, whether it matches or not provided amount. 
! At this point we cycle thru all the loop without changing anything.
! As in sage, here i/j refers to entire grid (1 to nlon/nlat), 
! ii/jj refers to subgrid (1 to nlonsub/nlatsub)
      nlpoints = 0
      do 5 j = jnorth, jsouth
         jj = j - jnorth + 1
         do 6 i = iwest, ieast
            ii = i - iwest + 1
#ifdef SINGLE_POINT_MODEL
            if ((ii*jj).eq.1) then
               lmask(ii,jj) = 1
            else
               lmask(ii,jj) = 0
            endif
#else /* SINGLE_POINT_MODEL */
            lmask(ii,jj) = nint(xmask(i,j))
#endif /* SINGLE_POINT_MODEL */
            if (lmask(ii,jj).eq.1) then
               nlpoints = nlpoints + 1
            end if
6        continue
5     continue
      if (nlpoints .ne. npoi) then
         write(*,9000)
         write(*,*) 'number of land points in input/surta.nc'
         write(*,*) 'does not match number of land points in inland_compar.h'
         write(*,*) 'in surta =',nlpoints,' in include/inland_compar.h =',npoi
         write(*,*) 'Replace this value in npoi parameter located in file include/inland_compar.h'
         write(*,*) 'Then, return to the main directory and re-compile the model with make dev-symlinks'
         stop 1
      else
         write (*,9010) 
         write (*,9020) nlpoints
         write (*,9010) 
      end if

      nlpoints = 0

! Now that 'npoi' structure fits the actual array, we can go ahead and
! write data to the structures.
! initialize lonindex, latindex for use in arr2vec, vec2arr, etc.
! and calculate the approximate the area of the land gridcells
! here, i/j refers to entire grid (1 to nlon/nlat), 
! ii/jj refers to subgrid (1 to nlonsub/nlatsub)
      do 7 j = jnorth, jsouth
         jj = j - jnorth + 1
         do 8 i = iwest, ieast
            ii = i - iwest + 1
#ifdef SINGLE_POINT_MODEL
            if ((ii*jj).eq.1) then
               lmask(ii,jj) = 1
            else
               lmask(ii,jj) = 0
            endif
#else /* SINGLE_POINT_MODEL */
            lmask(ii,jj) = nint(xmask(i,j))
#endif /* SINGLE_POINT_MODEL */
            if (lmask(ii,jj).eq.1) then
               nlpoints = nlpoints + 1
               lonindex(nlpoints) = ii
               latindex(nlpoints) = jj
               xlat = latscale(j) * pi / 180.0
               garea(nlpoints) = yres * 111400.0 * xres * 111400.0 * cos(xlat)
            end if
8        continue
7     continue

#ifdef SINGLE_POINT_MODEL
! NOTE NOTE NOTE ::: Single point model will return from here. Nothing below
!  this point will be executed!
      return

#else /* SINGLE_POINT_MODEL */
      do 9 j = jnorth, jsouth
         jj = j - jnorth + 1
         latscale(jj) = latscale(j)
9     continue
      do 10 i = iwest, ieast
         ii = i - iwest + 1
         lonscale(ii) = lonscale(i)
10    continue

! dummy variable example, 4-d, but 3rd dim ('level') = 1
! copy and chanve for a new variable
!
!      filen = 'input/dummyv.nc'
!      aname = 'dummyv'
!      call readvar(filen,aname,'level',istart,icount,
!     > cdummy,work(1),work(ndim4),work(2*ndim4),work(3*ndim4),istat)
!      if (istat.lt.0) then
!         write(*,9000)
!         print *, 'while reading dummyv'
!         stop 1
!      end if
!      call arr2vec(cdummy, xindummy)
!
! topography
      filen = 'input/topo.nc'
      aname = 'topo'

!   Remember ndim4 is the biggest between nlon,nlat,nband,nsoilay,nsnolay,npft;
! ndim2 (size of work) is nlon*nlat so as long as nlon and nlat are greater than
! or equal to 3, there will be no problem on the usage below. ndim2 is nlat*nlon
      call readvar(filen,aname,'level',istart,icount,cdummy,work(1),         &
                   work(ndim4),work(2*ndim4),work(3*ndim4),istat)
      if (istat.lt.0) then
         write(*,9000)
         print *, 'while reading topo'
         stop 1
      end if
      call arr2vec(cdummy, xintopo)

! fixed vegetation map
      if (isimveg .le. 1) then
         filen = 'input/vegtype.nc'
         aname = 'vegtype'
         call readvar(filen,aname,'level',istart,icount,cdummy,work(1),      &
                      work(ndim4),work(2*ndim4),work(3*ndim4),istat)
         if (istat.lt.0) then
            write(*,9000)
            print *, 'while reading vegtype'
            stop 1
         end if
         call arr2vec(cdummy, xinveg)
      end if

!c 2-d soil array
!c
!     filen = 'input/soil.nc'
!     aname = 'soil'
!     call readvar(filen,aname,'level',istart,icount,
!    > cdummy,work(1),work(ndim4),work(2*ndim4),work(3*ndim4),istat)
!     if (istat.lt.0) then
!        write(*,9000)
!        print *, 'while reading soil'
!        stop 1
!     end if
!     call arr2vec(cdummy, soita)
!
! delta t
      filen = 'input/deltat.nc'
      aname = 'deltat'
      call readvar(filen,aname,'level',istart,icount,cdummy,work(1),         &
                   work(ndim4),work(2*ndim4),work(3*ndim4),istat)
      if (istat.lt.0) then
         write(*,9000)
         print *, 'while reading deltat'
         stop 1
      end if
      call arr2vec(cdummy, deltat)

! 3-d soil texture array
!
! icount(3) is the 6 layers used in soita.sand.nc soita.clay.nc
      icount(3) = 6 
      icount(4) = 1
      filen = 'input/soita.sand.nc'
      aname = 'sandpct'
      call readvar(filen,aname,'layer',istart,icount,cdummy,work(1),         &
                   work(ndim4),work(2*ndim4),work(3*ndim4),istat)
      if (istat.lt.0) then
         write(*,9000)
         print *, 'while reading soita.sand'
         stop 1
      end if
      do 12 j = 1, nsoilay
         call arr2vec(cdummy((j-1)*nlonsub*nlatsub + 1), sand(1,j))
12    continue

      icount(3) = 6 
      icount(4) = 1
      filen = 'input/soita.clay.nc'
      aname = 'claypct'
      call readvar(filen,aname,'layer',istart,icount,cdummy,work(1),         &
                   work(ndim4),work(2*ndim4),work(3*ndim4),istat)
      if (istat.lt.0) then
         write(*,9000)
         print *, 'while reading soita.clay'
         stop 1
      end if
      do 13 j = 1, nsoilay
         call arr2vec(cdummy((j-1)*nlonsub*nlatsub + 1), clay(1,j))
13    continue

! 3-d climate arrays
      icount(3) = 1
      icount(4) = 12

      filen = 'input/wetd.mon.nc'
      aname = 'wetd'

      call readvar(filen,aname,'level',istart,icount,cdummy,work(1),   &
                   work(ndim4),work(2*ndim4),work(3*ndim4),istat)
      if (istat.lt.0) then
         write(*,9000)
         print *, 'while reading wetd'
         stop 1
      end if
      do 15 ntime = 1,12
         call arr2vec(cdummy((ntime-1)*nlonsub*nlatsub + 1),clmwet(1,ntime))
15    continue

      filen = 'input/temp.mon.nc'
      aname = 'temp'
      call readvar(filen,aname,'level',istart,icount,cdummy,work(1),         &
                   work(ndim4),work(2*ndim4),work(3*ndim4),istat)
      if (istat.lt.0) then
         write(*,9000)
         print *, 'while reading temp'
         stop 1
      end if
      do 20 ntime = 1,12
         call arr2vec(cdummy((ntime-1)*nlonsub*nlatsub + 1),clmt(1,ntime))
20    continue

      filen = 'input/trange.mon.nc'
      aname = 'trange'
      call readvar(filen,aname,'level',istart,icount,cdummy,work(1),         &
                   work(ndim4),work(2*ndim4),work(3*ndim4),istat)
      if (istat.lt.0) then
         write(*,9000)
         print *, 'while reading trange'
         stop 1
      end if
      do 25 ntime = 1,12
         call arr2vec(cdummy((ntime-1)*nlonsub*nlatsub + 1),clmtrng(1,ntime))
25    continue

      filen = 'input/prec.mon.nc'
      aname = 'prec'
      call readvar(filen,aname,'level',istart,icount,cdummy,work(1),         &
                   work(ndim4),work(2*ndim4),work(3*ndim4),istat)
      if (istat.lt.0) then
         write(*,9000)
         print *, 'while reading prec'
         stop 1
      end if
      do 30 ntime = 1,12
         call arr2vec(cdummy((ntime-1)*nlonsub*nlatsub + 1),clmprec(1,ntime))
30    continue

      filen = 'input/wspd.mon.nc'
      aname = 'wspd'
      call readvar(filen,aname,'level',istart,icount,cdummy,work(1),         &
                   work(ndim4),work(2*ndim4),work(3*ndim4),istat)
      if (istat.lt.0) then
         write(*,9000)
         print *, 'while reading wspd'
         stop 1
      end if
      do 35 ntime = 1,12
         call arr2vec(cdummy((ntime-1)*nlonsub*nlatsub + 1),xinwind(1,ntime))
35    continue

      filen = 'input/cld.mon.nc'
      aname = 'cld'
      call readvar(filen,aname,'level',istart,icount,cdummy,work(1),         &
                   work(ndim4),work(2*ndim4),work(3*ndim4),istat)
      if (istat.lt.0) then
         write(*,9000)
         print *, 'while reading cld'
         stop 1
      end if
      do 40 ntime = 1,12
         call arr2vec(cdummy((ntime-1)*nlonsub*nlatsub + 1),clmcld(1,ntime))
40    continue

      filen = 'input/rh.mon.nc'
      aname = 'rh'
      call readvar(filen,aname,'level',istart,icount,cdummy,work(1),         &
                   work(ndim4),work(2*ndim4),work(3*ndim4),istat)
      if (istat.lt.0) then
         write(*,9000)
         print *, 'while reading rh'
         stop 1
      end if
      do 45 ntime = 1,12
         call arr2vec(cdummy((ntime-1)*nlonsub*nlatsub + 1),clmq(1,ntime))
45    continue

! copy all 5 climatology fields to clm+anom fields for spin up
      ndim = npoi*12
      call scopya(ndim, clmt, xint)
      call scopya(ndim, clmtrng, xintrng)
      call scopya(ndim, clmprec, xinprec)
      call scopya(ndim, clmcld, xincld)
      call scopya(ndim, clmq, xinq)
      call scopya(ndim, clmwet, xinwet)

! return to main program
      return
#endif /* SINGLE_POINT_MODEL */

9000  format (1x,'ERROR in subroutine readit')
9010  format (1x,' ')
9020  format (1x,'number of land points: ', i10)

end subroutine readit
